/*
 * Nombre del proyecto: Proyecto Arkanoid
 *
 * Nombre del fichero: graph.c
 *
 * Nombre de los autores: Grzegorz Adam Hankiewicz y Carlos Pereda Moreno
 *
 * Fecha: 18 de Enero del 2000
 *
 * Descripcin: Rutinas grficas usadas para dibujar la paleta, la raqueta,
 * los ladrillos, los marcadores, etc.
 */

/* CABECERAS GLOBALES */
#ifdef MSS
#include <mss.h>
#endif MSS
#include <stdlib.h>
#include <string.h>
#include <allegro.h>


/* CABECERAS DE FUNCIONES EXTERNAS */
#include "general.h"
#include "struct.h"
#include "errores.h"
#include "log.h"
#include "lborrado.h"

/* CABECERA PROPIA */
#define DECLARA_VARIABLES_GRAPH_C
#include "graph.h"


/**[txh]********************************************************************

  Description: Dibuja la pelota en las coordenadas x_p e y_p. Esas mismas
  coordenadas se guardan en una lista al llamar a marca_zona_borrado para su
  posterior sobreescritura antes del siguiente redibujado.

***************************************************************************/

void     dibujar_pelota (DATOS_JUEGO *datos)
{
   if (!datos)
   {
      eprintf ("dibujar_pelota: puntero nulo");
      return;
   }

   draw_sprite (datos->bmp, datos->bola, (datos->x_p>>16) -
                datos->bola->w/2, (datos->y_p>>16) - datos->bola->h/2);

   marca_zona_borrado (lista_borrado_logica,
                       (datos->x_p>>16) - datos->bola->w/2,
                       (datos->y_p>>16) - datos->bola->h/2,
                       datos->bola->w, datos->bola->h);
}

/**[txh]********************************************************************

  Description: Dibuja la raqueta en la coordenada x_r. Esa misma coordenada
  se guarda en una lista al llamar a marca_zona_borrado para su posterior
  sobreescritura antes del siguiente redibujado

***************************************************************************/

void     dibujar_raqueta (DATOS_JUEGO *datos)
{
   if (!datos)
   {
      eprintf ("dibujar_raqueta: puntero nulo");
      return;
   }

   draw_sprite (datos->bmp, datos->raqueta, fixtoi (datos->x_r) -
                datos->raqueta->w/2, ALTURA_RAQUETA);

   marca_zona_borrado (lista_borrado_logica,
                       fixtoi (datos->x_r) - datos->raqueta->w/2,
                       ALTURA_RAQUETA, datos->raqueta->w, datos->raqueta->h);
}

/**[txh]********************************************************************

  Description: Imprime los puntos y las vidas en unas coordenadas fijas de
  la pantalla

***************************************************************************/

void     dibujar_marcador (BITMAP *bmp, DATOS_JUEGO *datos)
{
   int altura = SCREEN_H / 20;

   if (!bmp || !datos)
   {
      eprintf ("dibuja_marcador: punteros nulos");
      return;
   }

   /* Los puntos y las vidas los ponemos de distinto color que el fondo */
   escribe_texto (bmp, font, SCREEN_W-margen_der, altura, margen_der, altura,
                  makecol (0, 128, 255),  makecol (192, 192, 192), "Puntos:");

   escribe_texto (bmp, font, SCREEN_W-margen_der/2, altura*2, margen_der/2,
                  altura, makecol (0, 128, 255),  makecol (192, 192, 192),
                  "%05d", datos->puntos);

   escribe_texto (bmp, font, SCREEN_W-margen_der, altura*4, margen_der, altura,
                  makecol (0, 128, 255),  makecol (192, 192, 192), " Vidas:");

   escribe_texto (bmp, font, SCREEN_W-margen_der/8, altura*5, margen_der/8,
                  altura, makecol (0, 128, 255),  makecol (192, 192, 192),
                  "%d", datos->vidas);
}



/**[txh]********************************************************************

  Description: Dibuja los ladrillos en una pantalla virtual dependiendo del
  mapa de ladrillos. Para ello transformamos las coordenadas lgicas del
  bitmap de ladrillos en coordenadas fsicas para ajustar el tamao de los
  ladrillos en la pantalla. Tambin marcamos los bordes de cada ladrillo,
  comprobando que el color del ladrillo sea diferente. Se le pasa la
  estructura de datos y la zona de ladrillos que hay que redibujar.

***************************************************************************/


void     dibujar_ladrillos (DATOS_JUEGO *datos, int x1, int y1, int x2, int y2)
{
   int ix, iy, col;

   for (iy = y1; iy < y2; iy++)
   {
      for (ix = x1; ix < x2; ix++)
      {
         col = getpixel (datos->ladrillos, ix, iy);
         rectfill (datos->bmp, ix*TAM_CELDA, iy*TAM_CELDA, (ix+1)*TAM_CELDA-1,
                   (iy+1)*TAM_CELDA-1, color[col%16]);

         if (col != 0)
         {
            if (color_ladrillo_diferente (datos, ix+0, iy-1, col))
               hline (datos->bmp, ix*TAM_CELDA, iy*TAM_CELDA, (ix+1)*TAM_CELDA-1,
                      color[0]);
            if (color_ladrillo_diferente (datos, ix+0, iy+1, col))
               hline (datos->bmp, ix*TAM_CELDA, (iy+1)*TAM_CELDA-1, (ix+1)*TAM_CELDA-1,
                      color[0]);
            if (color_ladrillo_diferente (datos, ix-1, iy+0, col))
               vline (datos->bmp, ix*TAM_CELDA, iy*TAM_CELDA, (iy+1)*TAM_CELDA-1,
                      color[0]);
            if (color_ladrillo_diferente (datos, ix+1, iy+0, col))
               vline (datos->bmp, (ix+1)*TAM_CELDA-1, iy*TAM_CELDA, (iy+1)*TAM_CELDA-1,
                      color[0]);
         }
      }
   }
}


/**[txh]********************************************************************

  Description: Funcin que llama a las funciones de dibujado y redibujado de
  la raqueta y la pelota en bmp(pantalla virtual) y el marcador en la
  pantalla. Vsync evita que se produzca parpadeo en la pantalla. Blit
  transfiere todo lo que se ha dibujado en bmp a la pantalla.
  Recupera_zonas_pantalla redibuja en la pantalla la pelota y la raqueta.

***************************************************************************/


void     dibuja_pantalla (DATOS_JUEGO *datos)
{
   if (!datos)
   {
      eprintf ("dibuja_pantalla: puntero nulo");
      return;
   }

   recupera_zonas_borrado (datos);
   
   dibujar_pelota (datos);
   dibujar_raqueta (datos);

   vsync();
   dibujar_marcador (screen, datos);
   recupera_zonas_pantalla (screen, datos);
}

/**[txh]********************************************************************

  Description: Inicializamos los mdulos grficos de dibujado y redibujado
  del juego. 1 configuramos la resolucin de la pantalla, establecemos los
  mrgenes de la zona de la pantalla a la que transferimos el bitmap bmp y
  finalmente inicializamos las listas de borrado de coordenadas lgicas y
  fsicas y la tabla de colores para los ladrillos.

  Return: Devuelve 0 si se realiz todo con xito, distinto de 0 en caso
  contrario.
  
***************************************************************************/


int      inicializa_motor_grafico (void)
{
   int ret;

   escalado_pantalla = obten_escalado_optimo();
   if (!escalado_pantalla)
   {
      eprintf ("inicializa_motor_grafico: el juego necesita una resolucion "
               "de pantalla mayor para funcionar.");
      return 2;
   }

   margen_der = SCREEN_W/5;
   margen_alto = (SCREEN_H - (RES_LOGICA_Y * escalado_pantalla)) / 2;
   margen_izq = ((SCREEN_W - margen_der) - (RES_LOGICA_X * escalado_pantalla)) / 2;

   if (margen_alto < 0) margen_alto = 0;
   if (margen_izq < 0)
   {
      /* Compensamos los bordes de la pantalla */
      margen_der += margen_izq;
      margen_izq = 0;
   }

   /* Ajuste del margen derecho de pantalla, para los textos */
   if (SCREEN_W-(RES_LOGICA_X*escalado_pantalla+margen_izq) > margen_der)
      margen_der = SCREEN_W-(RES_LOGICA_X*escalado_pantalla+margen_izq);

   ret = inicializa_listas_borrado();
   if (ret)
      return ret;

   inicia_tabla_de_colores();
      
   return 0;
}


/**[txh]********************************************************************

  Description: Rellena el array de colores con la informacin adecuada para
  que el juego dibuje bien los ladrillos.

***************************************************************************/

void     inicia_tabla_de_colores (void)
{
   color[0] = makecol (0, 0, 0);
   color[1] = makecol (0, 0, 170);
   color[2] = makecol (0, 170, 0);
   color[3] = makecol (0, 170, 170);
   color[4] = makecol (255, 255, 0);
   color[5] = makecol (255, 0, 0);
   color[6] = makecol (0, 255, 0);
   color[7] = makecol (76, 191, 255);
   color[8] = makecol (255, 102, 255);
   color[9] = makecol (0, 0, 191);
   color[10] = makecol (255, 178, 128);
   color[11] = makecol (191, 128, 73);
   color[12] = makecol (128, 128, 128);
   color[13] = makecol (191, 191, 191);
   color[14] = makecol (255, 128, 25);
   color[15] = makecol (255, 153, 153);
}



/**[txh]********************************************************************

  Description: Llama a destruye_lista_borrado que "limpia" las listas de
  coordenadas fsicas y lgicas.

***************************************************************************/

void     finaliza_motor_grafico (void)
{
   destruye_lista_borrado ();
}

/**[txh]********************************************************************

  Description: Dibuja los contornos de la pantalla en los que no se
  dibujar nada de la pantalla virtual.

***************************************************************************/


void     dibuja_marco_pantalla (BITMAP *bmp)
{
   if (!bmp)
   {
      eprintf ("dibuja_marco_pantalla: Puntero nulo");
      return;
   }

   clear_to_color (bmp, makecol (192, 192, 192));
}

/**[txh]********************************************************************

  Description: Funcin que obtiene el escalado ptimo para dibujar la
  pantalla en un tamao adecuado.
  
  Return: Devuelve 0 si la resolucin de pantalla actual es demasiado pequea.
  Devuelve 1 o ms para indicar el escalado ptimo.
  
***************************************************************************/


int      obten_escalado_optimo (void)
{
   int ret = 1;
   
   while (RES_LOGICA_Y * ret <= SCREEN_H && RES_LOGICA_X * ret <= SCREEN_W)
      ret++;
      
   return ret-1;
}

/**[txh]********************************************************************

  Description: Funcin que comprueba si el ladrillo de la posicin x e y es
  del mismo color que le pasamos como parmetro.

  Return: Devuelve 0 si el ladrillo es del mismo color; 1 en caso contrario
  o en caso de que las coordenadas no estn dentro de la pantalla.

***************************************************************************/

int      color_ladrillo_diferente (DATOS_JUEGO *datos, int x, int y, int col)
{
   if (x < 0 || y < 0 || x >= datos->ladrillos->w || y >= datos->ladrillos->h)
      return 1;

   if (getpixel (datos->ladrillos, x, y) != col)
      return 1;

   return 0;
}



/**[txh]********************************************************************

  Description: Funcin que envuelve a @x{_escribe_texto}, permitiendo pasar
  un nmero de parmetros variables para formatear cadenas. Se pasa: el
  bitmap de destino, el tipo de letra, la posicin x, y en coordenadas de
  pantalla fsicas, el ancho,alto fsico, y el color. Si el color es
  negativo, el texto ser transparente.

  Example:
   escribe_texto (screen, font, 0, 0, 100, 20, 12, -1, "Tengo %d aos", 5);

***************************************************************************/


void     escribe_texto (BITMAP *bmp, FONT *font, int x, int y, int w, int h,
                        int color_texto, int color_fondo, char *texto, ...)
{
   char buf[STR];
   va_list ap;

   if (!bmp || !font)
   {
      eprintf ("escribe_texto: punteros nulos");
      return ;
   }

   /* Transformacin de parmetros en cadena simple */
   va_start (ap, texto);
   uvsprintf (buf, texto, ap);
   va_end (ap);

   _escribe_texto (bmp, font, x, y, w, h, color_texto, color_fondo, buf);
}

/**[txh]********************************************************************

  Description: Funcin que crea un bitmap temporal, imprime el texto, y
  reescala ese bitmap en el destino. Acepta parmetros como los de
  @x{escribe_texto}. Slo se llama internamente.

***************************************************************************/


void     _escribe_texto (BITMAP *bmp, FONT *font, int x, int y, int w, int h,
                         int color_texto, int color_fondo, char *texto)
{
   BITMAP *tmp;

   tmp = create_bitmap (2+text_length (font, texto), 2+ text_height (font));
   if (!tmp)
   {
      eprintf ("_escribe_texto: no hay memoria suficiente");
      return ;
   }

   /* Dibujamos el fondo del color requerido */
   if (color_fondo < 0)
      clear_to_color (tmp, bitmap_mask_color(tmp));
   else
      clear_to_color (tmp, color_fondo);

   /* Esto es para dibujar un borde negro */
   textout (tmp, font, texto, 2, 1, makecol (0, 0, 0));
   textout (tmp, font, texto, 0, 1, makecol (0, 0, 0));
   textout (tmp, font, texto, 1, 2, makecol (0, 0, 0));
   textout (tmp, font, texto, 1, 0, makecol (0, 0, 0));
   textout (tmp, font, texto, 1, 1, color_texto);

   /* Dibujamos el texto y salimos */
   if (color_fondo < 0)
      masked_stretch_blit (tmp, bmp, 0, 0, tmp->w, tmp->h, x, y, w, h);
   else
      stretch_blit (tmp, bmp, 0, 0, tmp->w, tmp->h, x, y, w, h);
      
   destroy_bitmap (tmp);
}

