/*
 * Nombre del proyecto: Proyecto Arkanoid
 *
 * Nombre del fichero: hiscore.c
 *
 * Nombre de los autores: Grzegorz Adam Hankiewicz y Carlos Pereda Moreno
 *
 * Fecha: 18 de Enero del 2000
 *
 * Descripcin:
 * Este cdigo esta extrado del juego SPEED, escrito por Shawn Hargreaves
 * en 1999 para la competicin de internet SpeedHack. La versin original
 * completa del juego se puede encontrar en http://www.talula.demon.co.uk/speed/
 * El programa es giftware, se puede hacer lo que sea con l, y como tal
 * hemos decidido hacer uso del cdigo de puntuaciones.
 *
 * Se han hecho cambios para amoldar las rutinas al juego:
 *  - Uso de funciones u* de Allegro en vez de sprintf, strcpy, etc.
 *  - Ligera reescritura de la funcin que muestra los records.
 *  - Cambio de records predefinidos.
 *  - Se aadi documentacin a las funciones.
 */

/* CABECERAS GLOBALES */
#ifdef MSS
#include <mss.h>
#endif MSS
#include <allegro.h>

/* CABECERAS DE FUNCIONES EXTERNAS */
#include "general.h"
#include "errores.h"
#include "main.h"

/* CABECERA PROPIA */
#define DECLARA_VARIABLES_HISCORE_C
#include "hiscore.h"


#define NUM_SCORES      8
#define MAX_NAME_LEN    24

/* the score table */
static int scores[NUM_SCORES] =
{
   666, 512, 440, 256, 192, 128, 64, 24
};

static char names[NUM_SCORES][MAX_NAME_LEN+1];

static char yourname[MAX_NAME_LEN+1] = "";



/**[txh]********************************************************************

  Description: Inicializa el sistema de errores, rellenando la tabla con
  valores predefinidos y creando el fichero de records.

***************************************************************************/

void     init_hiscore (void)
{
   char buf1[256];
   int i;

   /* Cambiamos el fichero de configuracin por el de records */
   push_config_state();
   set_config_file (fichero_records);

   for (i = 0; i < NUM_SCORES; i++)
   {
      usprintf (buf1, "score%d", i+1);
      scores[i] = get_config_int ("hiscore", buf1, scores[i]);

      usprintf (buf1, "name%d", i+1);
      ustrncpy (names[i], get_config_string ("hiscore", buf1, "Bill Gates"), MAX_NAME_LEN);
      names[i][MAX_NAME_LEN] = 0;
   }

   pop_config_state();
}


/**[txh]********************************************************************

  Description: Funcin que guarda en el fichero de rcords los records
  actuales.

***************************************************************************/


void     shutdown_hiscore (void)
{
   char buf1[256];
   int i;

   push_config_state();
   set_config_file (fichero_records);

   for (i = 0; i < NUM_SCORES; i++)
   {
      usprintf (buf1, "score%d", i+1);
      set_config_int ("hiscore", buf1, scores[i]);

      usprintf (buf1, "name%d", i+1);
      set_config_string ("hiscore", buf1, names[i]);
   }

   pop_config_state();
}


/**[txh]********************************************************************

  Description: Funcin de ayuda de @x{score_table} que muestra una caja
  enmarcada con un cursor para introducir el nombre del jugador en la
  tabla de rcords. Se le pasa un puntero a BITMAP y la posicin en la tabla
  donde ser dibujada la caja.

***************************************************************************/

void     draw_entry_box (BITMAP *bmp, int which)
{
   BITMAP *b = create_bitmap (MAX_NAME_LEN*8+16, 16);
   int x;

   clear_to_color (b, makecol (0, 96, 0));
   hline (b, 0, b->h-1, b->w, makecol (0, 32, 0));
   vline (b, b->w-1, 0, b->h, makecol (0, 32, 0));

   textprintf (b, font, 9, 5, makecol (0, 0, 0), "%s", yourname);
   textprintf (b, font, 8, 4, makecol (255, 255, 255), "%s", yourname);

   if (retrace_count & 8)
   {
      x = ustrlen (yourname)*8 + 8;
      rectfill (b, x, 12, x+7, 14, makecol(0, 0, 0));
   }

   blit (b, bmp, 0, 0, SCREEN_W/2-56, SCREEN_H/2+(which-NUM_SCORES/2)*16-4, b->w, b->h);

   destroy_bitmap(b);
}

/**[txh]********************************************************************

  Description: Funcin de tabla de rcords. Se le pasa un parmetro que es
  la puntuacin del jugador tras jugar una partida. La funcin determina
  si la puntuacin no es suficiente para inscribirse. En este caso,
  simplemente muestra la tabla de rcords hasta que se pulse una tecla.@*
  Si se ha superado un rcord, se muestra la tabla de rcords con un espacio
  enmarcado y un cursor parpadeante. Es ahora cuando el usuario simplemente
  teclea su nombre y aparecer en los rcords.

***************************************************************************/

void     score_table (int score)
{
   BITMAP *bmp, *b;
   int c, i, j, x, y;
   int myscore = -1;

   for (i = 0; i < NUM_SCORES; i++)
   {
      if (score > scores[i])
      {
         for (j = NUM_SCORES-1; j>i; j--)
         {
            scores[j] = scores[j-1];
            ustrcpy (names[j], names[j-1]);
         }

         scores[i] = score;
         ustrcpy (names[i], yourname);

         myscore = i;
         break;
      }
   }

   bmp = create_bitmap(SCREEN_W, SCREEN_H);

   if (bitmap_color_depth(bmp) > 8)
   {
      for (i = 0; i < SCREEN_W/2; i++)
      {
         vline (bmp, SCREEN_W/2-i-1, 0, SCREEN_H, makecol (0, i*255/(SCREEN_W/2), 0));
         vline (bmp, SCREEN_W/2+i, 0, SCREEN_H, makecol (0, i*255/(SCREEN_W/2), 0));
      }
   }
   else
      clear_to_color (bmp, makecol(0, 128, 0));

   b = create_bitmap (text_length (font, "TABLA DE RECORDS"), 8);
   clear_to_color (b, bitmap_mask_color (bmp));

   textout (b, font, "TABLA DE RECORDS", 0, 0, makecol (0, 0, 0));
   stretch_sprite (bmp, b, SCREEN_W/64+4, SCREEN_H/64+4, SCREEN_W*31/32, SCREEN_H/8);
   stretch_sprite (bmp, b, SCREEN_W/64+4, SCREEN_H*55/64+4, SCREEN_W*31/32, SCREEN_H/8);

   textout (b, font, "TABLA DE RECORDS", 0, 0, makecol (0, 64, 0));
   stretch_sprite (bmp, b, SCREEN_W/64, SCREEN_H/64, SCREEN_W*31/32, SCREEN_H/8);
   stretch_sprite (bmp, b, SCREEN_W/64, SCREEN_H*55/64, SCREEN_W*31/32, SCREEN_H/8);

   destroy_bitmap (b);

   for (i=0; i<NUM_SCORES; i++)
   {
      y = SCREEN_H/2 + (i-NUM_SCORES/2) * 16;

      textprintf (bmp, font, SCREEN_W/2-142, y+2, makecol (0, 0, 0), "#%d - %d", i+1, scores[i]);
      textprintf (bmp, font, SCREEN_W/2-47, y+1, makecol (0, 0, 0), "%s", names[i]);

      if (i == myscore)
         c = makecol (255, 0, 0);
      else
         c = makecol (255, 255, 255);

      textprintf (bmp, font, SCREEN_W/2-144, y, c, "#%d - %d", i+1, scores[i]);
      textprintf (bmp, font, SCREEN_W/2-48, y, c, "%s", names[i]);
   }

   if (myscore >= 0)
      draw_entry_box (bmp, myscore);

   c = retrace_count;

   for (i=0; i<=SCREEN_H/16; i++)
   {
      acquire_screen();

      for (j=0; j<=16; j++)
      {
         x = j*(SCREEN_W/16) + i;
         blit (bmp, screen, x, 0, x, 0, 1, SCREEN_H);

         y = j*(SCREEN_H/16) + i;
         blit (bmp, screen, 0, y, 0, y, SCREEN_W, 1);
      }

      release_screen();

      do {
      } while (retrace_count < c + i*512/SCREEN_W);
   }

   destroy_bitmap (bmp);

   while (joy_b1)
      poll_joystick();

   while ((key[KEY_SPACE]) || (key[KEY_ENTER]) || (key[KEY_ESC]))
      poll_keyboard();

   if (myscore >= 0)
   {
      clear_keybuf();

      for (;;)
      {
         poll_joystick();

         if ((joy_b1) && (yourname[0]))
         {
            ustrcpy (names[myscore], yourname);
            break;
         }

         if (keypressed())
         {
            c = readkey();

            if (((c >> 8) == KEY_ENTER) && (yourname[0]))
            {
               ustrcpy (names[myscore], yourname);
               break;
            }
            else if (((c >> 8) == KEY_ESC) && (names[myscore][0]))
            {
               ustrcpy (yourname, names[myscore]);
               break;
            }
            else if (((c >> 8) == KEY_BACKSPACE) && (ustrlen(yourname) > 0))
            {
               yourname[ustrlen(yourname)-1] = 0;
            }
            else if (((c & 0xFF) >= ' ') && ((c & 0xFF) <= '~') && (ustrlen(yourname) < MAX_NAME_LEN))
            {
               yourname[ustrlen(yourname)+1] = 0;
               yourname[ustrlen(yourname)] = (c & 0xFF);
            }
         }

         draw_entry_box(screen, myscore);
      }
   }
   else
   {
      while ((!key[KEY_SPACE]) && (!key[KEY_ENTER]) && (!key[KEY_ESC]) && (!joy_b1))
      {
         poll_joystick();
         poll_keyboard();
      }
   }
}




